definition module corba_wrappers

//
// This module contains wrapper functions around the ORB routines
//
// It is written by hand because htoclean could not process include files
// which contain C++ declarations.
// Most functions take an !*env argument, which is used to serialize the
// function calls. This is similar to the approach used by Object IO.
//

import StdEnv, StdDebug

from StdString import String

////////////////////////////////
//       CStringArray
////////////////////////////////

:: CStringArrayPtr 

c_string_array_create :: !Int !*env -> (!CStringArrayPtr, !*env)

c_string_array_set :: !CStringArrayPtr !Int !String !*env -> !*env

c_string_array_get :: !CStringArrayPtr !Int !*env -> (!String, !*env)

c_string_array_delete :: !CStringArrayPtr !*env -> !*env

////////////////////////////////
//         ORB
////////////////////////////////

:: ORBPtr 

corba_orb_init :: (!CStringArrayPtr, !Int) !*env -> (!ORBPtr, !CStringArrayPtr, !Int, !*env)

:: CorbaObjectPtr :== Int

// Returns 0 if the conversion failed
corba_orb_string_to_object :: !ORBPtr !String !*env -> (!CorbaObjectPtr, !*env)

// Returns 0 if the bind failed
corba_orb_bind :: !ORBPtr !String !String !*env -> (!CorbaObjectPtr, !*env)

// Returns "" if the conversion failed
corba_orb_object_to_string :: !ORBPtr !CorbaObjectPtr !*env -> (!String, !*env)

// Returns 0 if failed
corba_orb_resolve_initial_references :: !ORBPtr !String !*env -> (!CorbaObjectPtr, !*env)

corba_object_is_a :: !CorbaObjectPtr !String !*env -> (!Bool, !*env)

corba_object_nil :: !*env -> (!CorbaObjectPtr, !*env)

corba_object_hash :: !CorbaObjectPtr !*env -> (!Int, !*env)

/////////////////////////////////
//        TYPECODE
////////////////////////////////

// These should be equal to the TCKind values used by the ORB 
// implementation
TK_NULL 	:== 0
TK_VOID 	:== 1
TK_SHORT 	:== 2
TK_LONG 	:== 3
TK_USHORT 	:== 4
TK_ULONG 	:== 5
TK_FLOAT 	:== 6
TK_DOUBLE 	:== 7
TK_BOOLEAN 	:== 8
TK_CHAR 	:== 9
TK_OCTET 	:== 10
TK_ANY 		:== 11
TK_TYPECODE :== 12
TK_PRINCIPAL:== 13
TK_OBJREF	:== 14
TK_STRUCT	:== 15
TK_UNION	:== 16
TK_ENUM		:== 17
TK_STRING	:== 18
TK_SEQUENCE :== 19
TK_ARRAY	:== 20
TK_ALIAS	:== 21
TK_EXCEPT	:== 22

// The other typecodes (TK_FIXED etc.) are not supported

:: TypeCodePtr

// Aux structure used to construct IDL structures

:: StructMemberSeqPtr 

corba_struct_member_seq_create :: !Int !*env -> (!StructMemberSeqPtr, !*env)

corba_struct_member_seq_fill :: !StructMemberSeqPtr !Int !String !TypeCodePtr !*env -> !*env

corba_struct_member_seq_delete :: !StructMemberSeqPtr !*env -> !*env

// Aux structure used to construct IDL unions

:: UnionMemberSeqPtr 

corba_union_member_seq_create :: !Int !*env -> (!UnionMemberSeqPtr, !*env)

corba_union_member_seq_fill :: !UnionMemberSeqPtr !Int !String !AnyPtr !TypeCodePtr !*env -> !*env

corba_union_member_seq_delete :: !UnionMemberSeqPtr !*env -> !*env

// Aux structure used to construct IDL enums

:: EnumMemberSeqPtr 

corba_enum_member_seq_create :: !Int !*env -> (!EnumMemberSeqPtr, !*env)

corba_enum_member_seq_fill :: !EnumMemberSeqPtr !Int !String !*env -> !*env

corba_enum_member_seq_delete :: !EnumMemberSeqPtr !*env -> !*env

// Creation functions

// Create a TypeCode which does not require additional parameters
corba_tc_create_basic :: !Int !*env -> (!TypeCodePtr, !*env)

// Create a TypeCode of type TK_STRING
corba_tc_create_string :: !Int !*env -> (!TypeCodePtr, !*env)

corba_tc_create_objref :: !String !String !*env -> (!TypeCodePtr, !*env)

corba_tc_create_struct :: !String !String !StructMemberSeqPtr !*env -> (!TypeCodePtr, !*env)

corba_tc_create_sequence :: !Int !TypeCodePtr !*env -> (!TypeCodePtr, !*env)

corba_tc_create_union :: !String !String !TypeCodePtr !UnionMemberSeqPtr !*env -> (!TypeCodePtr, !*env)

corba_tc_create_enum :: !String !String !EnumMemberSeqPtr !*env -> (!TypeCodePtr, !*env)

corba_tc_create_alias :: !String !String !TypeCodePtr !*env -> (!TypeCodePtr, !*env)

corba_tc_create_array :: !TypeCodePtr !Int !*env -> (!TypeCodePtr, !*env)

corba_tc_create_exception :: !String !String !StructMemberSeqPtr !*env -> (!TypeCodePtr, !*env)

corba_tc_create_recursive :: !String !*env -> (!TypeCodePtr, !*env)

// Accessor functions

corba_tc_get_kind :: !TypeCodePtr !*env -> (!Int, !*env)

corba_tc_get_id :: !TypeCodePtr !*env -> (!String, !*env)

corba_tc_get_name :: !TypeCodePtr !*env -> (!String, !*env)

corba_tc_get_member_count :: !TypeCodePtr !*env -> (!Int, !*env)

corba_tc_get_member_name :: !TypeCodePtr !Int !*env -> (!String, !*env)

corba_tc_get_member_type :: !TypeCodePtr !Int !*env -> (!TypeCodePtr, !*env)

corba_tc_get_member_label :: !TypeCodePtr !Int !*env -> (!AnyPtr, !*env)

corba_tc_get_discriminator_type :: !TypeCodePtr !*env -> (!TypeCodePtr, !*env)

corba_tc_get_default_index :: !TypeCodePtr !*env -> (!Int, !*env)

corba_tc_get_length :: !TypeCodePtr !*env -> (!Int, !*env)

corba_tc_get_content_type :: !TypeCodePtr !*env -> (!TypeCodePtr, !*env)

////////////////////////////////
//          CORBA::Any
////////////////////////////////

:: AnyPtr 

//
// Creation functions
//

corba_any_create_null :: !*env -> (!AnyPtr, !*env)

corba_any_create_short :: !Int !*env -> (!AnyPtr, !*env)

corba_any_create_ushort :: !Int !*env -> (!AnyPtr, !*env)

corba_any_create_long :: !Int !*env -> (!AnyPtr, !*env)

corba_any_create_ulong :: !Int !*env -> (!AnyPtr, !*env)

corba_any_create_float :: !Real !*env -> (!AnyPtr, !*env)

corba_any_create_double :: !Real !*env -> (!AnyPtr, !*env)

corba_any_create_bool :: !Bool !*env -> (!AnyPtr, !*env)

corba_any_create_char :: !Char !*env -> (!AnyPtr, !*env)

corba_any_create_octet :: !Int !*env -> (!AnyPtr, !*env)

corba_any_create_string :: !String !*env -> (!AnyPtr, !*env)

corba_any_create_any :: !AnyPtr !*env -> (!AnyPtr, !*env)

corba_any_create_tc :: !TypeCodePtr !*env -> (!AnyPtr, !*env)

corba_any_create_objref :: !CorbaObjectPtr !*env -> (!AnyPtr, !*env)

corba_any_delete :: !AnyPtr !*env -> !*env

//
// Extraction functions
//

corba_any_extract_short :: !AnyPtr !*env -> (!Int, !*env)

corba_any_extract_long :: !AnyPtr !*env -> (!Int, !*env)

corba_any_extract_ushort :: !AnyPtr !*env -> (!Int, !*env)

corba_any_extract_ulong :: !AnyPtr !*env -> (!Int, !*env)

corba_any_extract_float :: !AnyPtr !*env -> (!Real, !*env)

corba_any_extract_double :: !AnyPtr !*env -> (!Real, !*env)

corba_any_extract_boolean :: !AnyPtr !*env -> (!Bool, !*env)

corba_any_extract_char :: !AnyPtr !*env -> (!Char, !*env)

corba_any_extract_octet :: !AnyPtr !*env -> (!Int, !*env)

corba_any_extract_string :: !AnyPtr !*env -> (!String, !*env)

corba_any_extract_any :: !AnyPtr !*env -> (!AnyPtr, !*env)

corba_any_extract_tc :: !AnyPtr !*env -> (!TypeCodePtr, !*env)

corba_any_extract_objref :: !AnyPtr !*env -> (!CorbaObjectPtr, !*env)

// Obtain the TypeCode of the Any
corba_any_get_type :: !AnyPtr !*env -> (!TypeCodePtr, !*env)

//
// DynStruct interface
//

:: NameValuePairSeq 

corba_nv_pair_seq_create :: !Int !*env -> (!NameValuePairSeq, !*env)

// Fill a member of the sequence
corba_nv_pair_seq_fill :: !NameValuePairSeq !Int !String !AnyPtr !*env -> !*env

corba_nv_pair_seq_length :: !NameValuePairSeq !*env -> (!Int, !*env)

corba_nv_pair_seq_get :: !NameValuePairSeq !Int !*env -> (!String, !AnyPtr, !*env)

corba_nv_pair_seq_delete :: !NameValuePairSeq !*env -> !*env

// Answers the Any extracted from the DynStruct
corba_dyn_struct_create :: !TypeCodePtr !NameValuePairSeq !*env -> (!AnyPtr, !*env)

// Answer the member list extracted from the DnyStruct
corba_dyn_struct_members :: !AnyPtr !*env -> (!NameValuePairSeq, !*env)

//
// DynSequence Interface
//

:: AnySeq 

corba_any_seq_create :: !Int !*env -> (!AnySeq, !*env)

// Fill a member of the sequence
corba_any_seq_fill :: !AnySeq !Int !AnyPtr !*env -> !*env

corba_any_seq_length :: !AnySeq !*env -> (!Int, !*env)

corba_any_seq_get :: !AnySeq !Int !*env -> (!AnyPtr, !*env)

corba_any_seq_delete :: !AnySeq !*env -> !*env

// Answers the Any extracted from the DynSequence
corba_dyn_sequence_create :: !TypeCodePtr !AnySeq !*env -> (!AnyPtr, !*env)

// Answer the contents of the DnySequence
corba_dyn_sequence_elements :: !AnyPtr !*env -> (!AnySeq, !*env)

//
// DynUnion Interface
//

// Answers the Any extracted from the DynUnion
corba_dyn_union_create :: !TypeCodePtr !AnyPtr !AnyPtr !*env -> (!AnyPtr, !*env)

// Answer the discriminator and the active member of the DynUnion
// If there is no active member, and Any with typecode tk_null is returned
corba_dyn_union_get :: !AnyPtr !*env -> (!AnyPtr, !AnyPtr, !*env)

//
// DynEnum Interface
//

// Answers the Any extracted from the DynEnum
corba_dyn_enum_create :: !TypeCodePtr !String !*env -> (!AnyPtr, !*env)

// Answer the value of the DynEnum
corba_dyn_enum_get :: !AnyPtr !*env -> (!String, !*env)

//
// DynArray Interface
//

// Answers the Any extracted from the DynArray
corba_dyn_array_create :: !TypeCodePtr !AnySeq !*env -> (!AnyPtr, !*env)

// Answer the contents of the DnyArray
corba_dyn_array_elements :: !AnyPtr !*env -> (!AnySeq, !*env)

////////////////////////////////
//       EXCEPTION
////////////////////////////////

:: ExceptionPtr :== Int

// These should be equal to the CompletionStatus values used by the ORB 
// implementation
COMPLETED_YES 	:== 0
COMPLETED_NO 	:== 1
COMPLETED_MAYBE	:== 2

corba_exception_is_system :: !ExceptionPtr !*env -> (!Bool, !*env)

// Only for system exceptions
corba_exception_get_minor :: !ExceptionPtr !*env -> (!Int, !*env)

// Only for system exceptions
corba_exception_get_completed :: !ExceptionPtr !*env -> (!Int, !*env)

corba_exception_get_repoid :: !ExceptionPtr !*env -> (!String, !*env)

:: UnknownExPtr :== Int

// Narrow the exception to UnknownUserException, or return 0
corba_unknown_ex_narrow :: !ExceptionPtr !*env -> (!UnknownExPtr, !*env)

// Answer the repoid of the user exception contained in the exception
corba_unknown_ex_get_repoid :: !UnknownExPtr !*env -> (!String, !*env)

// Try to decode the contents of the Exception based on the TypeCode
corba_unknown_ex_get_value :: !UnknownExPtr !TypeCodePtr !*env -> (!AnyPtr, !*env)

////////////////////////////////
//        NAMEDVALUE
////////////////////////////////

:: NamedValuePtr 

// Values for the Flags field in the NamedValue object
// These should be equal to the ArgModeFlags values used by the ORB 
// implementation

:: ArgModeFlags :== Int

ARG_IN 		:== 1
ARG_OUT 	:== 2
ARG_INOUT	:== 4

corba_nv_get_name :: !NamedValuePtr !*env -> (!String, !*env)

corba_nv_get_value :: !NamedValuePtr !*env -> (!AnyPtr, !*env)

corba_nv_get_flags :: !NamedValuePtr !*env -> (!ArgModeFlags, !*env)

// TODO: Memory management
corba_nv_set_type :: !NamedValuePtr !TypeCodePtr !*env -> !*env

// TODO: Memory management
corba_nv_set_value :: !NamedValuePtr !AnyPtr !*env -> !*env

////////////////////////////////
//         NVLIST
////////////////////////////////

:: NVListPtr 

//corba_nvlist_add :: !NVListPtr !String !AnyPtr !Int !*env -> (!NVListPtr, !*env)

corba_nvlist_create :: !*env -> (!NVListPtr, !*env)

corba_nvlist_add :: !NVListPtr !ArgModeFlags !*env -> (!NamedValuePtr, !*env)

corba_nvlist_count :: !NVListPtr !*env -> (!Int, !*env)

corba_nvlist_get :: !NVListPtr !Int !*env -> (!NamedValuePtr, !*env)

////////////////////////////////
//        REQUEST
////////////////////////////////

:: RequestPtr 

corba_object_create_request :: !CorbaObjectPtr !String !*env -> (!RequestPtr, !*env)

corba_request_arguments :: !RequestPtr !*env -> (!NVListPtr, !*env)

corba_request_add_in_arg :: !RequestPtr !String !AnyPtr !*env -> !*env

corba_request_add_out_arg :: !RequestPtr !String !AnyPtr !*env -> !*env

corba_request_add_inout_arg :: !RequestPtr !String !AnyPtr !*env -> !*env

corba_request_add_out_arg_template :: !RequestPtr !String !TypeCodePtr !*env -> !*env

corba_request_invoke :: !RequestPtr !*env -> !*env

// Set the type of expected result
corba_request_set_result_tc :: !RequestPtr !TypeCodePtr !*env -> !*env

// The returned AnyPtr is owned by the Request
corba_request_result :: !RequestPtr !*env -> (!AnyPtr, !*env)

// Answer the raised exception or 0
corba_request_exception :: !RequestPtr !*env -> (!ExceptionPtr, !*env)

// Add a possible exception
corba_request_add_exception_def :: !RequestPtr !TypeCodePtr !*env -> !*env

corba_request_delete :: !RequestPtr !*env -> !*env

////////////////////////////////
//     SERVER REQUEST
////////////////////////////////

:: ServerRequestPtr

// Return the operation name of the request
corba_server_request_operation :: !ServerRequestPtr !*env -> (!String, !*env)

// Set the arguments of the request
// The caller must fill in the types of arguments
// The ORB will fill in the values of the arguments
// The ServerRequest assumes ownership of the NVList
corba_server_request_set_params :: !ServerRequestPtr !NVListPtr !*env -> !*env

// Set the result of the request
corba_server_request_set_result :: !ServerRequestPtr !AnyPtr !*env -> !*env

// Set the result of the request
// ????????????????
corba_server_request_set_exception :: !ServerRequestPtr !AnyPtr !*env -> !*env

corba_server_request_get_args :: !ServerRequestPtr !*env -> (!NVListPtr, !*env)

////////////////////////////////
//      SERVER SIDE
////////////////////////////////

corba_listener_init :: !*env -> !*env

// The returned ServerRequestPtr and CorbaObjectPtr are owned by the ORB,
// and must not be freed by the application
corba_listener_receive :: !*env -> (!ServerRequestPtr, !CorbaObjectPtr, !*env)

// Signal the ORB that the reply has been computed
corba_listener_reply :: !*env -> !*env

corba_servant_create :: !String !*env -> (!CorbaObjectPtr, !*env)

// Local Variables:
// tab-width: 4
// End:




