
#include "show_corba.h"

// ----------------------------------------------------------------------
// Get original type, i.e. remove aliases
// ----------------------------------------------------------------------

// ORBacus has a built in function with the same name. Strange...
#ifdef CLEAN_ORB_MICO
CORBA2(TypeCode_ptr)
GetOrigType(CORBA2(TypeCode_ptr) p)
{
/*
    CORBA2(TypeCode_var) tc = CORBA2(TypeCode)::_duplicate(p);

    while(tc -> kind() == CORBA2(tk_alias))
	tc = tc -> content_type();

    return tc._retn();
*/
  return CORBA2(TypeCode)::_duplicate ( p->unalias() );
}
#endif

// ----------------------------------------------------------------------
// display CORBA::TypeCode
// ----------------------------------------------------------------------

static char* NameTypeIDL [] = { "null", "void", "short", "long",
  "unsigned short", "unsigned long", "float", "double", "boolean", "char",
  "octet", "any", "TypeCode", "Principal", "objref", "struct", "union",
  "enum", "string", "sequence", "array", "alias", "exception"
};

void CORBA_OUT::display_IDL_TypeCode (CORBA2(TypeCode_ptr) tc)
{
    CORBA2(ULong) i;
    CORBA2(TCKind) kind = tc->kind();

    switch ( kind ) {
    case CORBA2(tk_objref):
        skip() << "interface " << tc->name() << ';';
        break;

    case CORBA2(tk_struct):
    case CORBA2(tk_except): {
        skip() << NameTypeIDL[kind] << ' ' << tc->name() << " {" << endl;

        down ();
        for (i=0; i<tc->member_count(); i++) {
            skip();
            CORBA2(TypeCode_var) member_type = tc->member_type(i);
            show_name_TypeCode (member_type);
            ostr_ << ' ' << tc->member_name(i) << ';' << endl;
        }
        up ();
        skip() << "};";
      } break;
 
    case CORBA2(tk_union): {
        skip () << "union " << tc->name() << " switch(";
        CORBA2(TypeCode_var) discriminator_type = tc->discriminator_type();
        show_name_TypeCode (discriminator_type);
        ostr_ << ") {" << endl;

        down ();
        for (i=0 ; i<tc->member_count(); i++) {
            CORBA2(Any_var) any_member_label = tc->member_label(i);
            CORBA2(TypeCode_var) type_label = any_member_label->type();
            if ( type_label->kind() == CORBA2(tk_octet) ) {
	        skip() << "default";
            } else {
	        skip() << "case ";
                CORBA2_DynAny_var dyn_any_member_label =
		    fac_->create_dyn_any (any_member_label);
               show_DynAny_without_type (dyn_any_member_label);
            }
            ostr_ << ": ";
            CORBA2(TypeCode_var) member_type = tc->member_type(i);
            show_name_TypeCode (member_type);
            ostr_ << ' ' << tc->member_name(i) << ';' << endl;
        }
        up();
        skip() << "};"; 
      } break;

    case CORBA2(tk_enum):
        skip()  << "enum " << tc->name() << " {";
        for (i=0; i<tc->member_count(); i++) {
            if ( i != 0 )
 	       ostr_ << ", ";
            ostr_ << tc->member_name(i);
        }
        ostr_ << "};";
        break;

    case CORBA2(tk_string): {
        skip() << "string";
        CORBA2(ULong) length = tc->length();
        if (length != 0)
	  ostr_ << '<' << length << '>';
      } break;

    case CORBA2(tk_sequence): {
        skip() << "sequence<";
        CORBA2(TypeCode_var) content_type = tc->content_type();
        show_name_TypeCode (content_type);
        CORBA2(ULong) length = tc->length();
        if (length != 0)
          ostr_ << ", " << length;
        ostr_ << '>';
      } break;

    case CORBA2(tk_array): {
        skip();
        CORBA2(TypeCode_var) content_type = tc->content_type();
        show_name_TypeCode (content_type);
        ostr_ << '[' << tc->length() << ']';
      } break;

    case CORBA2(tk_alias): {
        skip() << "typedef ";
        CORBA2(TypeCode_var) content_type = tc->content_type();
        show_name_TypeCode (content_type);
        ostr_ << ' ' << tc->name() << ';';
      } break;

    default:
        skip() << NameTypeIDL [kind];
            break;
    }
    ostr_ << endl;
}

void CORBA_OUT::show_name_TypeCode (CORBA2(TypeCode_ptr) tc)
{
    CORBA2(TCKind) kind = tc->kind();

    switch ( kind ) {
    case CORBA2(tk_objref):
    case CORBA2(tk_enum):
    case CORBA2(tk_union):
    case CORBA2(tk_struct):
    case CORBA2(tk_except):
    case CORBA2(tk_alias):
        ostr_ << tc->name();
        break;

    case CORBA2(tk_string): {
        ostr_ << "string";
        CORBA2(ULong) length = tc->length();
        if (length != 0)
	  ostr_ << '<' << length << '>';
      } break;

    case CORBA2(tk_sequence): {
        ostr_ << "sequence<";
        CORBA2(TypeCode_var) content_type = tc->content_type();
        show_name_TypeCode (content_type);
        CORBA2(ULong) length = tc->length();
        if (length != 0)
          ostr_ << ", " << length;
        ostr_ << '>';
      } break;

    case CORBA2(tk_array): {
        CORBA2(TypeCode_var) content_type = tc->content_type();
        show_name_TypeCode (content_type);
        ostr_ << '[' << tc->length() << ']';
      } break;

    default:
        ostr_ << NameTypeIDL [kind];
        break;
    }
}

// ----------------------------------------------------------------------
// show CORBA::DynAny
// ----------------------------------------------------------------------

void CORBA_OUT::display_DynAny (CORBA2_DynAny_ptr dyn_any)
{
    skip();
    show_DynAny(dyn_any);
    ostr_ << endl;
}

void CORBA_OUT::show_DynAny (CORBA2_DynAny_ptr dyn_any)
{
    CORBA2(TypeCode_var) tc = dyn_any->type();

    show_name_TypeCode (tc);
    ostr_ << '(';
    show_DynAny_without_type(dyn_any);
    ostr_ << ')';
}

void CORBA_OUT::show_DynAny_without_type (CORBA2_DynAny_ptr dyn_any)
{
    CORBA2(ULong) i;

    dyn_any->rewind ();

    CORBA2(TypeCode_var) tc = dyn_any->type();
    tc = GetOrigType(tc);

    CORBA2(TCKind) kind = tc->kind();

    switch ( kind ) {
    case CORBA2(tk_void):
	break;

	case CORBA2(tk_objref): {
	} break;

    case CORBA2(tk_short): {
        CORBA2(Short) v_short = dyn_any->get_short();
        ostr_ << v_short;
      } break;

    case CORBA2(tk_ushort): {
        CORBA2(UShort) v_ushort = dyn_any->get_ushort();
        ostr_ << v_ushort;
      } break;

    case CORBA2(tk_long): {
        CORBA2(Long) v_long = dyn_any->get_long();
        ostr_ << v_long;
      } break;

    case CORBA2(tk_ulong): {
        CORBA2(ULong) v_ulong = dyn_any->get_ulong();
        ostr_ << v_ulong;
      } break;

    case CORBA2(tk_float): {
        CORBA2(Float) v_float = dyn_any->get_float();
        ostr_ << v_float;
      } break;

    case CORBA2(tk_double): {
        CORBA2(Float) v_double = dyn_any->get_double();
        ostr_ << v_double;
      } break;

    case CORBA2(tk_boolean): {
        CORBA2(Boolean) v_boolean = dyn_any->get_boolean();
//        if ( v_boolean == CORBA2(TRUE) ) ostr_ << "TRUE";
//        else if ( v_boolean == CORBA2(FALSE) ) ostr_ << "FALSE";
//        else ostr_ << "UNKNOWN_BOOLEAN";
        if ( v_boolean ) ostr_ << "TRUE";
        else  ostr_ << "FALSE";
      } break;

    case CORBA2(tk_octet): {
        CORBA2(Octet) v_octet = dyn_any->get_octet();
        ostr_ << int(v_octet);
      } break;

    case CORBA2(tk_char): {
        CORBA2(Char) v_char = dyn_any->get_char();
        ostr_ << "'" << (char)v_char << "'";
      } break;

    case CORBA2(tk_string): {
        CORBA2(String_var) v_string = dyn_any->get_string();
        ostr_ << '"' << v_string.in() << '"';
      } break;

    case CORBA2(tk_TypeCode): {
        CORBA2(TypeCode_var) v_tc = dyn_any->get_typecode();
        show_name_TypeCode (v_tc);
      } break;

    case CORBA2(tk_any): {
        CORBA2(Any_var) v_any = dyn_any->get_any();
        show_Any(v_any);
      } break; 

    case CORBA2(tk_array): {
        CORBA2_DynArray_var dyn_array = CORBA2_DynArray::_narrow (dyn_any);
        for (i=0; i<tc->length(); i++) {
            if ( i != 0 ) ostr_ << ", ";
            CORBA2_DynAny_var component = dyn_array->current_component();
            show_DynAny (component);
            dyn_array->next ();
	}
      } break;   

    case CORBA2(tk_sequence): {
        CORBA2_DynSequence_var dyn_sequence =
	    CORBA2_DynSequence::_narrow (dyn_any);
        for (i=0; i<dyn_sequence->get_length(); i++) {
            if ( i != 0 ) ostr_ << ", ";
            CORBA2_DynAny_var component = dyn_sequence->current_component();
            show_DynAny (component);
            dyn_sequence->next ();
	}
      } break;   

    case CORBA2(tk_enum): {
         CORBA2_DynEnum_var dyn_enum = CORBA2_DynEnum::_narrow (dyn_any);
         CORBA2(String_var) v_string = dyn_enum->get_as_string();
         ostr_ << v_string.in();
       } break;   

    case CORBA2(tk_struct): {
        CORBA2_DynStruct_var dyn_struct =
	    CORBA2_DynStruct::_narrow (dyn_any);

        ostr_ << endl;
        down ();

        CORBA2(ULong) member_count = tc->member_count();
        for (i=0; i<member_count; i++) {
            CORBA2(String_var) member_name = dyn_struct->current_member_name();
            skip() << member_name.in() << " = ";
            CORBA2_DynAny_var component = dyn_struct->current_component();
            show_DynAny (component);
            ostr_ << endl;
            dyn_struct->next();  
        }

        up ();
        skip();
      } break;

   case CORBA2(tk_union): {
        CORBA2_DynUnion_var dyn_union = CORBA2_DynUnion::_narrow(dyn_any);

        ostr_ << endl;
        down ();

        CORBA2_DynAny_var component = dyn_union->current_component();

        skip() << "discriminator = ";
        show_DynAny (component);
        ostr_ << endl;

        dyn_union->next();

        component = dyn_union->current_component();

        CORBA2(String_var) member_name = dyn_union->member_name ();
        skip() << member_name.in() << " = ";

        if ( !CORBA2(is_nil) (component) )
            show_DynAny (component);
        else
            ostr_ << "not initialised" << endl;

        ostr_ << endl;

        up ();
        skip();
      } break;

    case CORBA2(tk_except): {
        CORBA2_DynStruct_var dyn_struct =
	    CORBA2_DynStruct::_narrow (dyn_any);

        ostr_ << endl;
        down ();

        CORBA2(ULong) member_count = tc->member_count();
        for (i=0; i<member_count; i++) {
            CORBA2(String_var) member_name = dyn_struct->current_member_name();
            skip() << member_name.in() << " = ";
            CORBA2_DynAny_var component = dyn_struct->current_component();
            show_DynAny (component);
            ostr_ << endl;
            dyn_struct->next();  
        }

        up ();
        skip();
      } break;

    default:
        assert (false);
        break;
    }
    dyn_any->rewind();
}

// ----------------------------------------------------------------------
// show CORBA::Any
// ----------------------------------------------------------------------

void CORBA_OUT::display_Any (const CORBA2(Any)& any)
{
    CORBA2_DynAny_var dyn_any = fac_->create_dyn_any (any);
    display_DynAny (dyn_any);
}

void CORBA_OUT::show_Any (const CORBA2(Any)& any)
{
    CORBA2_DynAny_var dyn_any = fac_->create_dyn_any (any);
    show_DynAny (dyn_any);
}

// ----------------------------------------------------------------------
// show CORBA::AnySeq
// ----------------------------------------------------------------------

void
CORBA_OUT::display_AnySeq (CORBA2_AnySeq* any_seq)
{
  for (CORBA2(ULong) i=0; i<any_seq->length(); i++) {
      ostr_ << "  ";
      show_Any((*any_seq)[i]);
  }
  ostr_ << endl;
}


