module test_client

// import test

import StdEnv
import Corba

import RWSDebug, Debug

////////////////////////////////
//           TESTS
////////////////////////////////

call_test_args obj env
	= corba_invoke obj "test_args" [(InArg "arg1" (CORBA_String__encode "foo")), (OutArg "arg2" (CORBA_String__encode "")), (InOutArg "arg3" (CORBA_Long__encode 11))] CORBA_Long__tc [] env

call_echo obj env 
	= corba_invoke obj "echo" [(InArg "what" (CorbaString "foo"))] (Tk_String 0) [] env

call_echo_tc obj env 
	= corba_invoke obj "echo_tc" [(InArg "what" (CorbaTypeCode tc))] (Tk_TypeCode) [] env
where
	tc = (Tk_Struct "bla" "foo" [("aa", Tk_Long), ("bb", (Tk_String 0))])

call_deposit obj env 
	= corba_invoke obj "deposit" [(InArg "amount" (CorbaLong 10))] Tk_Void [] env

call_self obj env 
	= corba_invoke obj "self" [] (Tk_ObjRef "xxx" "xxx") [] env

call_balance obj env 
	= corba_invoke obj "balance" [] Tk_Long [] env

call_balance2 obj env 
	= corba_invoke obj "balance2" [(OutArgTemplate "balance" Tk_Long)] Tk_Void [] env

call_echo_struct obj env = corba_invoke obj "echo_struct" [(InArg "what" (CorbaStruct tc members))] tc [] env
	where
		tc = (Tk_Struct "bla" "foo" [
			("aaa", Tk_Short),
			("bbb", (Tk_Long)), 
			("ccc", (Tk_String 0)), 
			("eee", (Tk_Sequence 0 (Tk_Long))),
			("ggg", CORBA_Object__tc)
			])
		members = [
			("aaa", (CorbaShort 5)),
			("bbb", (CorbaLong 2)), 
			("ccc", (CorbaString "3")),
			("eee", (CorbaSequence (Tk_Sequence 0 (Tk_Long)) [(CorbaLong 1), (CorbaLong 2)])),
			("ggg", (CorbaObjRef obj))
			]

call_echo_seq obj env = corba_invoke obj "echo_seq" [(InArg "what" (CorbaSequence tc members))] tc [] env
where
	tc = (Tk_Sequence 0 (Tk_Long))
	members = [(CorbaLong 0), (CorbaLong 1), (CorbaLong 2)]

call_echo_union obj env = corba_invoke obj "echo_union" [(InArg "what" (CorbaUnion tc (CorbaShort 0) (CorbaShort 2)))] tc [] env
where
	tc = (Tk_Union "foo" "bar" (Tk_Short) [("aaa", (CorbaShort 0), (Tk_Short)), ("bbb", (CorbaShort 1), (Tk_Long))])

call_echo_union_default obj env = corba_invoke obj "echo_union" [(InArg "what" (CorbaUnion tc (CorbaShort 3) (CorbaChar 'a')))] tc [] env
where
	tc = (Tk_Union "foo" "bar" (Tk_Short) [("aaa", (CorbaShort 0), (Tk_Short)), ("bbb", (CorbaShort 1), (Tk_Long)), ("bbb", (CorbaShort 2), (Tk_Long)), ("ccc", (CorbaOctet 0), (Tk_Char))])

call_echo_enum obj env = corba_invoke obj "echo_enum" [(InArg "what" (CorbaEnum tc "SECOND"))] tc [] env
where
	tc = Tk_Enum "bla" "foo" ["FIRST", "SECOND", "THIRD"]

call_echo_array obj env = corba_invoke obj "echo_array" [(InArg "what" value)] tc [] env
where
	tc = (Tk_Array 2 member_tc)
	member_tc = (Tk_Array 3 Tk_Long)
	value = (CorbaArray tc [
		(CorbaArray member_tc [(CorbaLong 1), (CorbaLong 2), (CorbaLong 3)]),
		(CorbaArray member_tc [(CorbaLong 4), (CorbaLong 5), (CorbaLong 6)])
				])

call__get_an_attribute obj env
	= corba_invoke obj "_get_an_attribute"[](Tk_String 0) [] env

call__set_an_attribute obj env
	= corba_invoke obj "_set_an_attribute" [
		(InArg "attr" (CORBA_String__encode "foo"))
		] CORBA_Void__tc [] env


call_raise_exception obj env
	= corba_invoke obj "raise_exception" [] (Tk_Void) [(Tk_Exception "IDL:Useless/Useless_Exception:1.0" "Useless_Exception" [("member1", Tk_Long), ("member2", Tk_Short)])] env

call_echo_recursive obj env
	= corba_invoke obj "echo_recursive" [(InArg "what" e1)] tc [] env
//	= Useless_echo_recursive obj e1 env
	where
		tc = Tk_Struct "IDL:Useless/Recursive_struct:1.0" "Recursive_struct" [
			("aaa", CORBA_Short__tc),
			("bbb", nested_tc)
			]
		nested_tc = Tk_Struct "IDL:Useless/Recursive_struct/Nested:1.0" "Nested"  [("ccc", nested_member_tc)]
		nested_member_tc = Tk_Sequence 0 tc

		e1 = CorbaStruct tc [("aaa", (CorbaShort 1)), ("bbb", (CorbaStruct nested_tc [("ccc", (CorbaSequence nested_member_tc [e2, e3]))]))]

		e2 = CorbaStruct tc [("aaa", (CorbaShort 4)), ("bbb", (CorbaStruct nested_tc [("ccc", (CorbaSequence nested_member_tc []))]))]

		e3 = CorbaStruct tc [("aaa", (CorbaShort 3)), ("bbb", (CorbaStruct nested_tc [("ccc", (CorbaSequence nested_member_tc [e4]))]))]

		e4 = CorbaStruct tc [("aaa", (CorbaShort 2)), ("bbb", (CorbaStruct nested_tc [("ccc", (CorbaSequence nested_member_tc []))]))]

test_invoke obj env
	# (res, ex, out, env) = call_self obj env 		->> "self..."
	# (res, ex, out, env) = call_test_args obj env  ->> "test_args..."
	# (res, ex, out, env) = call_echo_tc obj env 	->> "echo_tc..."
	# (res, ex, out, env) = call_deposit obj env 	->> "deposit..."
	# (res, ex, out, env) = call_balance obj env 	->> "balance..."
	# (res, ex, out, env) = call_balance2 obj env 	->> "balance2..."
	# (res, ex, out, env) = call_echo_struct obj env ->> "echo_struct..."
	# (res, ex, out, env) = call_echo_seq obj env	->> "echo_seq..."
	# (res, ex, out, env) = call_echo_union obj env	->>	"echo_union..."
	# (res, ex, out, env) = call_echo_union_default obj env	->>	"echo_union_default..."
	# (res, ex, out, env) = call_echo_enum obj env	->>	"echo_enum..."
	# (res, ex, out, env) = call_echo_recursive obj env ->> "echo_recursive..."
	# (res, ex, out, env) = call__set_an_attribute obj env	->>	"set_an_attribute..."
	# (res, ex, out, env) = call__get_an_attribute obj env	->>	"get_an_attribute..."	
	# (res, ex, out, env) = call_raise_exception obj env	->>	"raise_exception..."
	= ("OK.", env)


////////////////////////////////
//          MAIN
////////////////////////////////

Start w
	# (orb,_, w)	= CORBA_ORB_init ["xxx"] w
//	# (Just obj, w) = CORBA_ORB_bind orb "IDL:Useless:1.0" "inet:localhost:4242" w
	# (Just obj, w) = ReadIORFromFile orb "test.ior" w
	= test_invoke obj w
//	= call_raise_exception obj w	->>	"raise_exception..."

// Local Variables:
// tab-width: 4
// End:




